#include <string.h>
#include <unistd.h>
#include <sys/types.h>
#include <sys/wait.h>
#include <errno.h>
#include <signal.h>

#include "common.h"

int stop_received = 0;
int pipe_out = 0;
int pipe_err = 0;

void stop_handler(int dummy) {
	stop_received = 1;
}

void usage() {
	die("usage: pipedto [-o] [-e] dest ... \\; source ...");
}

int main(int argc, char **argv) {
	int child_pipe[3];
	int source, dest;
	int code = 0;
	char **p;

	setprogname("pipedto");

	while (1) {
		char c = getopt(argc, argv, "+oe");
		if (c == -1)
			break;

		switch (c) {
		case 'o':
			pipe_out = 1;
			break;
		case 'e':
			pipe_err = 1;
			break;
		default:
			usage();
		}
	}

	if (!(pipe_out || pipe_err))
		die("at least one of -o or -e must be specified");

	if ((argc - optind) < 3)
		usage();
	
	p = &argv[optind];
	while (1) {
		if (*p == NULL)
			usage();
		if (strcmp(*p, ";") == 0)
			break;
		p++;
	}
	*p++ = NULL;
	if (*p == NULL)
		usage();

	if (pipe(child_pipe) < 0)
		dies("pipe failed");
	child_pipe[2] = -1;

	set_signal_handler(SIGINT, stop_handler);
	set_signal_handler(SIGTERM, stop_handler);

	dest = spawn_child(&argv[optind], child_pipe[0], -1, -1, child_pipe);

	source = spawn_child(p, -1,
		pipe_out ? child_pipe[1] : -1,
		pipe_err ? child_pipe[1] : -1,
		child_pipe);

	close(child_pipe[0]);
	close(child_pipe[1]);

	while (1) {
		int status, pid;

		if (dest == -1 && source == -1)
			break;

		pid = wait(&status);
		if (pid < 0 && errno != ECHILD && errno != EINTR)
			dies("wait failed");

		if (pid == source) {
			if (WIFEXITED(status))
				code = WEXITSTATUS(status);
			source = -1;
		} else if (pid == dest) {
			dest = -1;
		}

		if (stop_received) {
			kill_child(source, SIGTERM);
			kill_child(dest, SIGTERM);
			break;
		}
	}

	return code;
}

